# -----------------------------------------------------------------------------
# Copyright (c) 2013, The Deblur Development Team.
#
# Distributed under the terms of the BSD 3-clause License.
#
# The full license is in the file LICENSE, distributed with this software.
# -----------------------------------------------------------------------------

from collections import defaultdict
import re

import numpy as np

trans_dict = defaultdict(lambda: 5, A=0, C=1, G=2, T=3)
trans_dict['-'] = 4


class Sequence(object):
    """Sequence object to represent the aligned reads

    Attributes
    ----------
    label : str
        The sequence label
    sequence : str
        The sequence string
    length : int
        The sequence length (aligned)
    unaligned_length : int
        The unaligned sequence length
    frequency : float
        The number of times the sequence have been seen in the dataset
    np_sequence : numpy array of int8
        An int8 numpy representation of the sequence string

    Methods
    -------
    to_fasta
    """

    def __init__(self, label, sequence):
        self.label = label
        self.sequence = sequence.upper()
        self.length = len(self.sequence)
        self.unaligned_length = self.length - self.sequence.count('-')
        self.frequency = float(re.search('(?<=size=)\w+', self.label).group(0))
        self.np_sequence = np.array(
            [trans_dict[b] for b in self.sequence], dtype=np.int8)

    def __eq__(self, other):
        return (type(self) == type(other) and
                self.sequence == other.sequence and
                self.frequency == other.frequency)

    def __ne__(self, other):
        return not self.__eq__(other)

    def to_fasta(self):
        """Returns a string with the sequence in fasta format

        Returns
        -------
        str
            The FASTA representation of the sequence
        """
        prefix, suffix = re.split('(?<=size=)\w+', self.label, maxsplit=1)
        new_count = int(round(self.frequency))
        new_label = "%s%d%s" % (prefix, new_count, suffix)
        return ">%s\n%s\n" % (new_label, self.sequence)



# -----------------------------------------------------------------------------
# Copyright (c) 2015, The Deblur Development Team.
#
# Distributed under the terms of the BSD 3-clause License.
#
# The full license is in the file LICENSE, distributed with this software.
# -----------------------------------------------------------------------------

from unittest import TestCase, main
from shutil import rmtree
from tempfile import mkdtemp
from os.path import join, dirname, abspath
import logging
from os import makedirs, remove

from deblur.parallel_deblur import parallel_deblur
from deblur.workflow import (build_index_sortmerna,
                             start_log, sequence_generator)


class parallelDeblurTests(TestCase):
    """Test parallel deblur methods functionality.
    """

    def setUp(self):
        """ Create working directory and two FASTA input
            files corresponding to two samples (s1 and s2).
            Each input file contains 120 sequences, of which
            100 are 16S amplicons (ART simulator), 10 are
            chimeric sequences (Grinder) and 10 are PhiX
            artifacts (ART). The 100 amplicon sequences
            intend to evenly represent a community of 10
            species.
        """
        # test output can be written to this directory
        self.working_dir = mkdtemp()
        deblur_working_dir = join(self.working_dir, "deblur_working_dir")
        makedirs(deblur_working_dir)

        # the data directory for the workflow test files
        self.test_data_dir = join(dirname(abspath(__file__)), 'data')
        self.seqs_s1_fp = join(self.test_data_dir, 'seqs_s1.fasta')
        self.seqs_s2_fp = join(self.test_data_dir, 'seqs_s2.fasta')
        self.seqs_s3_fp = join(self.test_data_dir, 'seqs_s3.fasta')
        self.orig_s1_fp = join(self.test_data_dir, 'simset.s1.fasta')
        self.orig_s2_fp = join(self.test_data_dir, 'simset.s2.fasta')
        self.orig_s3_fp = join(self.test_data_dir, 'simset.s3.fasta')

        self.files_to_remove = []

        logfilename = join(self.working_dir, "log.txt")
        start_log(level=logging.DEBUG, filename=logfilename)

    def tearDown(self):
        for f in self.files_to_remove:
            remove(f)
        rmtree(self.working_dir)

    def compare_result(self, simfilename, origfilename, trim_length):
        """Compare the results of deblurring to the original mixture

        Parameters
        ----------
        simfilename : str
            name of the simulated reads fasta file
        origfilename : str
            name of the fasta file with the ground truth sequences
        trim_length : int
            the length used for trimming the sequences in deblurring
        """

        # get the trimmed ground truth sequences
        orig_seqs = [item[1] for item in sequence_generator(origfilename)]
        orig_seqs = [item[:trim_length].upper() for item in orig_seqs]

        # get the deblurred fasta file sequences
        out_seqs = [item[1] for item in sequence_generator(simfilename)]
        out_seqs = [item.upper() for item in out_seqs]

        out_seqs.sort()
        orig_seqs.sort()

        # test we see all ground truth sequences and no other
        self.assertEqual(out_seqs, orig_seqs)

    def test_parallel_deblur(self):
        """Test parallel deblur using 3 simulated sequence files.
        seqs1 - 100 reads using art, original sequences are >0.5 identical.
        seqs2 - 200 reads using grinder, original sequences are >0.9 identical,
        0.1 chimeras, 35 phix reads
        seqs3 - simple - 15 reads from seqs1 (10 reads for 1001203,
        5 reads for 694276) for manual test validation
        """
        # index the 70% rep. set database
        ref_fp = join(self.test_data_dir, '70_otus.fasta')
        ref_db_fp = build_index_sortmerna(
            ref_fp=(ref_fp,),
            working_dir=self.working_dir)

        trim_length = 100
        params = ['deblur', 'workflow', '--seqs-fp', 'ignorethis',
                  '--output-dir', self.working_dir, '--pos-ref-fp', ref_fp,
                  '-d', '1,0.06,0.02,0.02,0.01,0.005,0.005,0.005,0.001'
                  ',0.001,0.001,0.0005',
                  '-t', str(trim_length)]
        parallel_deblur([self.seqs_s1_fp, self.seqs_s2_fp, self.seqs_s3_fp],
                        params, ref_db_fp, None, jobs_to_start=2)

        deblur_working_dir = join(self.working_dir, "deblur_working_dir")

        deb1res = join(deblur_working_dir,
                       'seqs_s1.fasta.trim.derep.no_artifacts'
                       '.msa.deblur.no_chimeras')
        self.compare_result(deb1res, self.orig_s1_fp, trim_length=trim_length)

        deb2res = join(deblur_working_dir,
                       'seqs_s2.fasta.trim.derep.no_artifacts'
                       '.msa.deblur.no_chimeras')
        self.compare_result(deb2res, self.orig_s2_fp, trim_length=trim_length)

        deb3res = join(deblur_working_dir,
                       'seqs_s3.fasta.trim.derep.no_artifacts'
                       '.msa.deblur.no_chimeras')
        self.compare_result(deb3res, self.orig_s3_fp, trim_length=trim_length)


if __name__ == '__main__':
    main()



# -----------------------------------------------------------------------------
# Copyright (c) 2013, The Deblur Development Team.
#
# Distributed under the terms of the BSD 3-clause License.
#
# The full license is in the file LICENSE, distributed with this software.
# -----------------------------------------------------------------------------

from unittest import TestCase, main

import numpy as np
import numpy.testing as npt

from deblur.sequence import Sequence


class SequenceTests(TestCase):
    def setUp(self):
        self.label = "151_4447;size=1812;"
        self.sequence = "---aggatgcgagatgcgtggt-----"
        self.exp_seq = "---AGGATGCGAGATGCGTGGT-----"
        self.exp_np_seq = np.array([4, 4, 4, 0, 2, 2, 0, 3, 2, 1, 2, 0, 2, 0,
                                    3, 2, 1, 2, 3, 2, 2, 3, 4, 4, 4, 4, 4])

    def test_init(self):
        obs = Sequence(self.label, self.sequence)

        self.assertEqual(obs.label, self.label)
        self.assertEqual(obs.sequence, self.exp_seq)
        self.assertEqual(obs.length, 27)
        self.assertEqual(obs.unaligned_length, 19)
        self.assertEqual(obs.frequency, 1812)
        npt.assert_equal(obs.np_sequence, self.exp_np_seq)

    def test_init_uppercase(self):
        sequence = "---AGGATGCGAGATGCGTGGT-----"

        obs = Sequence(self.label, sequence)

        self.assertEqual(obs.label, self.label)
        self.assertEqual(obs.sequence, self.exp_seq)
        self.assertEqual(obs.length, 27)
        self.assertEqual(obs.unaligned_length, 19)
        self.assertEqual(obs.frequency, 1812)
        npt.assert_equal(obs.np_sequence, self.exp_np_seq)

    def test_init_mixed_case(self):
        sequence = "---AggATgcGAgatGCgtgGT-----"

        obs = Sequence(self.label, sequence)

        self.assertEqual(obs.label, self.label)
        self.assertEqual(obs.sequence, self.exp_seq)
        self.assertEqual(obs.length, 27)
        self.assertEqual(obs.unaligned_length, 19)
        self.assertEqual(obs.frequency, 1812)
        npt.assert_equal(obs.np_sequence, self.exp_np_seq)

    def test_init_non_actg_chars(self):
        sequence = "---FoOatgcgagatgcgtfOo-----"
        exp_seq = "---FOOATGCGAGATGCGTFOO-----"
        exp_np_seq = np.array([4, 4, 4, 5, 5, 5, 0, 3, 2, 1, 2, 0, 2, 0,
                               3, 2, 1, 2, 3, 5, 5, 5, 4, 4, 4, 4, 4])

        obs = Sequence(self.label, sequence)

        self.assertEqual(obs.label, self.label)
        self.assertEqual(obs.sequence, exp_seq)
        self.assertEqual(obs.length, 27)
        self.assertEqual(obs.unaligned_length, 19)
        self.assertEqual(obs.frequency, 1812)
        npt.assert_equal(obs.np_sequence, exp_np_seq)

    def test_eq_true(self):
        s1 = Sequence(self.label, self.sequence)
        s2 = Sequence(self.label, self.sequence)
        self.assertTrue(s1 == s2)

        s2 = Sequence("152_4447;size=1812;", self.sequence)
        self.assertTrue(s1 == s2)

    def test_eq_false(self):
        s1 = Sequence(self.label, self.sequence)
        s2 = Sequence(self.label, "---aggatgcgagatgcgtgtt-----")
        self.assertFalse(s1 == s2)

        self.assertFalse(s1 == 2)
        self.assertFalse(s1 == "---aggatgcgagatgcgtgtt-----")

    def test_ne_true(self):
        s1 = Sequence(self.label, self.sequence)
        s2 = Sequence(self.label, "---aggatgcgagatgcgtgtt-----")
        self.assertTrue(s1 != s2)

        self.assertTrue(s1 != 2)
        self.assertTrue(s1 != "---aggatgcgagatgcgtgtt-----")

    def test_ne_false(self):
        s1 = Sequence(self.label, self.sequence)
        s2 = Sequence(self.label, self.sequence)
        self.assertFalse(s1 != s2)

        s2 = Sequence("152_4447;size=1812;", self.sequence)
        self.assertFalse(s1 != s2)

    def test_to_fasta(self):
        seq = Sequence(self.label, self.sequence)
        obs = seq.to_fasta()
        exp = ">151_4447;size=1812;\n---AGGATGCGAGATGCGTGGT-----\n"
        self.assertEqual(obs, exp)

        seq.frequency = 1811.1
        obs = seq.to_fasta()
        exp = ">151_4447;size=1811;\n---AGGATGCGAGATGCGTGGT-----\n"
        self.assertEqual(obs, exp)

        seq.frequency = 1811.5
        obs = seq.to_fasta()
        exp = ">151_4447;size=1812;\n---AGGATGCGAGATGCGTGGT-----\n"
        self.assertEqual(obs, exp)


if __name__ == '__main__':
    main()



from unittest import TestCase, main
from os.path import join, abspath, dirname

from deblur.workflow import _system_call, create_otu_table
from tempfile import mkdtemp
from shutil import rmtree
from biom import load_table


class TestScript(TestCase):
    def setUp(self):
        self.data_dir = join(dirname(abspath(__file__)), 'data')
        self.output_dir = mkdtemp()

    def tearDown(self):
        rmtree(self.output_dir)

    def test_workflow(self):
        cmd = ["deblur", "workflow",
               "--seqs-fp", join(self.data_dir, 'usecase_mixedchars',
                                 'split'),
               "--output-dir", self.output_dir,
               "--error-dist", ("1, 0.06, 0.02, 0.02, 0.01, 0.005, 0.005, "
                                "0.005, 0.001, 0.001, 0.001, 0.0005"),
               '--indel-max', "3",
               '--indel-prob', "0.01",
               '--jobs-to-start', "1",
               '--mean-error', "0.005",
               '--min-reads', "0",
               '--min-size', "2",
               '--threads-per-sample', "1",
               '--trim-length', "100",
               '-w',
               ]
        # run deblur
        sout, serr, res = _system_call(cmd)

        # test that all feature sequences are all upper case
        with open(join(self.output_dir, 'all.seqs.fa'), 'r') as f:
            for line in f.readlines():
                if line.startswith('>'):
                    self.assertEqual(line[1:].strip(),
                                     line[1:].strip().upper())

    def test_create_otu_table(self):
        fp_table = join(self.output_dir, 'all.biom')
        create_otu_table(
            fp_table,
            [(join(self.data_dir, 'usecase_mixedchars',
                   ('1.SKB7.640196.fastq.trim.derep.'
                    'no_artifacts.msa.deblur.no_chimeras')), '1.SKB7.640196'),
             (join(self.data_dir, 'usecase_mixedchars',
                   ('1.SKB8.640193.fastq.trim.derep.'
                    'no_artifacts.deblur.no_chimeras')), '1.SKB8.640193')],
            outputfasta_fp=join(self.output_dir, 'all.seqs'), minreads=0)
        table = load_table(fp_table)

        # should produce a table with two samples and two features
        self.assertEqual(table.shape, (2, 2))

        # assert that counts from different case entries are collapsed
        self.assertTrue(list(table.to_dataframe().to_dense().loc[
            ('TACGGGGGGGGTTAGCGTTATTCAATGATATTTGGCGTAAAGTGCATGTAGATGGTGTTAC'
             'AAGTTAAAAAAATAAAAACTAAGGACAAATCTTTTCGTT'), :].values) == [60, 0])


if __name__ == "__main__":
    main()



# -----------------------------------------------------------------------------
# Copyright (c) 2015, The Deblur Development Team.
#
# Distributed under the terms of the BSD 3-clause License.
#
# The full license is in the file LICENSE, distributed with this software.
# -----------------------------------------------------------------------------

from unittest import TestCase, main

from deblur.support_files import pos_db, neg_db

import os.path


class supportFilesTests(TestCase):
    """Test the supporting data files
    """
    def test_reference_(self):
        """Test if the positive and negative filtering
        reference fasta files exist
        """
        # the positive filtering fasta file
        self.assertTrue(os.path.isfile(pos_db))
        # the negative filtering fasta file
        self.assertTrue(os.path.isfile(neg_db))


if __name__ == '__main__':
    main()
